<?php

/**
 * ClouDNS DNS Manager v1.0
 */

if (!defined("WHMCS")) {
    die("This file cannot be accessed directly");
}

function cloudns_MetaData() {
    return array(
        'DisplayName' => 'ClouDNS DNS Manager',
        'APIVersion' => '1.1',
        'RequiresServer' => false,
    );
}

function cloudns_ConfigOptions($params) {
	$cloudns = Cloudns_Core::inst($params);
	$servers = $cloudns->Servers->getAvailableServers();
	$serverString = '';
	foreach ($servers as $name) {
		$serverString .= $name."<br />";
	}
	return array(
		'Zones' => array(
			'Type' => 'text',
			'Size' => '25',
			'Default' => '0',
			'Description' => 'How many zones are supported by this plan/product',
		),
		'Servers' => array(
			'Type' => 'textarea',
			'Rows' => 10,
			'Cols' => 50,
			'Description' => "List of DNS server for the customers to use. One server per row. Server names only. <br />Here is an example, with the default ones:<br /><br />".$serverString."<br /> If the list is empty and you didn't specify global servers the default servers of your ClouDNS account will be shown",
			'Default' => '',
		),
	);
}


require_once dirname(__FILE__) . "/cloudns_core/core.php";

function cloudns_CreateAccount ($params) {
	$cloudns = Cloudns_Core::inst($params);
	
	try {
		
	} catch (Exception $e) {
		// Record the error in WHMCS's module log.
		logModuleCall(
			'cloudns',
			__FUNCTION__,
			$params,
			$e->getMessage(),
			$e->getTraceAsString()
		);
		return $e->getMessage();
	}
	
	return 'success';
}


function cloudns_SuspendAccount ($params) {
	$cloudns = Cloudns_Core::inst($params);
	
	try {
		// make the zones inactive
		$result = $cloudns->Actions->suspendZones();
	} catch (Exception $e) {
		// Record the error in WHMCS's module log.
		logModuleCall(
			'cloudns',
			__FUNCTION__,
			$params,
			$e->getMessage(),
			$e->getTraceAsString()
		);
		return $e->getMessage();
	}
	
	return $result;
}


function cloudns_UnsuspendAccount ($params) {
	$cloudns = Cloudns_Core::inst($params);
	try {
		// make the zones active again
		$result = $cloudns->Actions->suspendZones();
	} catch (Exception $e) {
		// Record the error in WHMCS's module log.
		logModuleCall(
			'cloudns',
			__FUNCTION__,
			$params,
			$e->getMessage(),
			$e->getTraceAsString()
		);
		return $e->getMessage();
	}
	return $result;
}


function cloudns_TerminateAccount ($params) {
	$cloudns = Cloudns_Core::inst($params);
	try {
		// delete the zones
		$result = $cloudns->Actions->terminateAccount();
	} catch (Exception $e) {
		// Record the error in WHMCS's module log.
		logModuleCall(
			'cloudns',
			__FUNCTION__,
			$params,
			$e->getMessage(),
			$e->getTraceAsString()
		);
		return $e->getMessage();
	}
	return $result;
}
function cloudns_ChangePackage ($params) {
	$cloudns = Cloudns_Core::inst($params);
	
	try {
		
	} catch (Exception $e) {
		// Record the error in WHMCS's module log.
		logModuleCall(
			'cloudns',
			__FUNCTION__,
			$params,
			$e->getMessage(),
			$e->getTraceAsString()
		);
		return $e->getMessage();
	}
	
	return 'success';
}


function cloudns_ClientArea ($params) {
	$cloudns = Cloudns_Core::inst($params);
	$requestedAction = isset($_REQUEST['customAction']) ? $_REQUEST['customAction'] : 'zones';
	
	try {
		$zone = $cloudns->Helper->getRequest('zone');
		$response = $zoneInfo = $recordTypes = array();
		
		if (!empty($zone)) {
			$zoneExists = $cloudns->Controller->zoneExists($zone);
			if (isset($zoneExists['status']) && ($requestedAction != 'add-existing-zone' && $requestedAction != 'add-zone')) {
				// the zone doesn't exists in ClouDNS
				if ($zoneExists['status'] == 'error') {
					throw new Exception($zoneExists['description']);
				// the zone doesn't exists here
				} elseif ($zoneExists['status'] == 'info') {
					$cloudns->Helper->redirect("clientarea.php?action=productdetails&id={$params['serviceid']}");
				}
			}
		}
		
		if ($requestedAction == 'add-zone') {
			$zoneType = $cloudns->Helper->getPost('zoneType');
			
			$servers = $cloudns->Helper->getPost('masterDNSServer');
			$option = $cloudns->Helper->getPost('newZoneOptions', 1);
			$sufix = $cloudns->Helper->getPost('zoneSufix');
			$masterIP = $cloudns->Helper->getPost('slaveMasterIp');
			
			if (!empty($sufix)) {
				$zone = $zone.'.'.$sufix;
			}
			
			$response = $cloudns->Controller->addNewZone($zone, $zoneType, $option, $servers, $masterIP);
			
			if ($response['status'] == 'success') {
				$cloudns->Helper->redirect("clientarea.php?action=productdetails&id={$params['serviceid']}&customAction=zone-settings&zone={$response['zone']}");
			} else {
				$version = explode('.', $params['whmcsVersion']);
				$templateVariables = array(
					'response' => $response,
					'serversList' => $cloudns->Servers->getAvailableServers(),
					'version'=>$version[0],
					'theme' => $params['clientareatemplate'],
				);
			}
			
			$templateFile = 'templates/add-new-zone.tpl';
			
		} elseif ($requestedAction == 'add-existing-zone') {
			$response = $cloudns->Controller->addNewExistingZone($zone);
			if ($response['status'] == 'success') {
				$cloudns->Helper->redirect("clientarea.php?action=productdetails&id={$params['serviceid']}&customAction=zone-settings&zone={$response['zone']}");
			} else {
				$version = explode('.', $params['whmcsVersion']);
				$templateVariables = array(
					'response' => $response,
					'cloudAction' => 'add-new-zone',
					'version'=>$version[0],
					'theme' => $params['clientareatemplate'],
					
				);
			}
			$templateFile = 'templates/add-new-zone.tpl';
			
		} elseif ($requestedAction == 'zone-settings') {
			
			$zoneInfo = $cloudns->Zones->getZoneInfo($zone);
			$recordTypes = $cloudns->Controller->getRecordTypes($zoneInfo['zone']);
			$type = 'all';
			if (in_array($cloudns->Helper->getPost('recordsType'), $recordTypes)) {
				$type = $cloudns->Helper->getPost('recordsType');
			} elseif (in_array($cloudns->Helper->getGet('type'), $recordTypes)) {
				$type = $cloudns->Helper->getGet('type');
			}
			
			if ($zoneInfo['type'] == 'slave') {
				$templateFile = 'templates/slave/master-servers.tpl';
			} else {
				$templateFile = 'templates/records.tpl';
			}
			
			
			$templateVariables = $cloudns->Actions->getSettings($zoneInfo, $type, $record_id);
			
			if (isset($templateVariables['response']) && isset($templateVariables['response']['status']) && $templateVariables['response']['status'] == 'error') {
				$templateFile = 'templates/zone-error.tpl';
			}
			
		} elseif ($requestedAction == 'delete-record') {
			$zoneInfo = $cloudns->Zones->getZoneInfo($zone);
			$record_id = $cloudns->Helper->getGet('record');
			$templateVariables = $cloudns->Actions->deleteRecord($zoneInfo, $record_id);
			$templateFile = 'templates/records.tpl';
			
		} elseif ($requestedAction == 'delete-zone') {
			$templateVariables = $cloudns->Actions->deleteZone($zone);
			$templateFile = 'templates/zones.tpl';
			
		} elseif ($requestedAction == 'bind-settings') {
			$zoneInfo = $cloudns->Zones->getZoneInfo($zone);
			$templateVariables = $cloudns->Actions->getBINDSettings($zoneInfo);
			$templateFile = 'templates/slave/slave-bind-settings.tpl';
			
		} elseif ($requestedAction == 'delete-master-servers') {
			$master = $cloudns->Helper->getGet('master_server_id');
			$templateVariables = $cloudns->Actions->deleteMasterServer($zone, $master);
			$templateFile = 'templates/slave/master-servers.tpl';
			
		} elseif ($requestedAction == 'add-master-servers') {
			$master = $cloudns->Helper->getPost('masterIP');
			$templateVariables = $cloudns->Actions->addMasterServer($zone, $master);
			$templateFile = 'templates/slave/master-servers.tpl';
		
		} elseif ($requestedAction == 'add-new-record') {
			$templateVariables = $cloudns->Actions->addNewRecord($zone, $cloudns->Helper->getRequest('type'));
			$templateFile = 'templates/add-new-record.tpl';
			
		} elseif ($requestedAction == 'add-record') {
			$recordType = $cloudns->Helper->getRequest('addRecordType');
			$settings = array();
			$settings['host'] = $cloudns->Helper->getPost('addRecordHost');
			$settings['record'] = html_entity_decode($cloudns->Helper->getPost('addRecordRecord'));
			$settings['ttl'] = $cloudns->Helper->getPost('addRecordTtl');
			$status = 1;
			
			if ($recordType == 'SRV') {
				$settings['priority'] = $cloudns->Helper->getPost('addRecordSRVPriority');
				$settings['weight'] = $cloudns->Helper->getPost('addRecordWeight');
				$settings['port'] = $cloudns->Helper->getPost('addRecordPort');
			} else if ($recordType == 'MX') {
				$settings['priority'] = $cloudns->Helper->getPost('addRecordMXPriority');
			} else if ($recordType == 'WR') {
				$settings['frame'] = $cloudns->Helper->getPost('addRecordWRFrame');
				$settings['frame-title'] = $cloudns->Helper->getPost('addRecordWRFrameTitle');
				$settings['frame-description'] = $cloudns->Helper->getPost('addRecordWRFrameDescription');
				$settings['frame-keywords'] = $cloudns->Helper->getPost('addRecordWRFrameKeywords');
				$settings['save-path'] = $cloudns->Helper->getPost('addRecordWRSavePath');
				$settings['redirect-type'] = $cloudns->Helper->getPost('wr_type');
			} else if ($recordType == 'RP') {
				$settings['mail'] = $cloudns->Helper->getPost('addRecordMail');
				$settings['txt'] = $cloudns->Helper->getPost('addRecordTxt');
			} else if ($recordType == 'SSHFP') {
				$settings['algorithm'] = $cloudns->Helper->getPost('algorithm');
				$settings['fptype'] = $cloudns->Helper->getPost('fp_type');
			} else if ($recordType == 'NAPTR') {
				$settings['order'] = $cloudns->Helper->getPost('addRecordOrder');
				$settings['pref'] = $cloudns->Helper->getPost('addRecordPref');
				$settings['flag'] = $cloudns->Helper->getPost('flag');
				$settings['params'] = $cloudns->Helper->getPost('addRecordParams');
				$settings['regexp'] = $cloudns->Helper->getPost('addRecordRegexp');
				$settings['replace'] = $cloudns->Helper->getPost('addRecordReplace');
			}
			
			$templateVariables = $cloudns->Actions->doAddNewRecord($zone, $recordType, $settings);
			$templateFile = 'templates/add-new-record.tpl';
			
		} elseif ($requestedAction == 'edit-record') {
			$record_id = $cloudns->Helper->getGet('dns_record_id');
			$templateVariables = $cloudns->Actions->editRecord($zone, $record_id);
			$templateFile = 'templates/edit-record.tpl';
			
		} elseif ($requestedAction == 'do-edit-record') {
			$record_id = $cloudns->Helper->getGet('dns_record_id');
			$recordType = $cloudns->Helper->getPost('recordType');
			
			$settings = array();
			$settings['host'] = $cloudns->Helper->getPost('editRecordHost');
			$settings['record'] = html_entity_decode($cloudns->Helper->getPost('editRecordRecord'));
			$settings['ttl'] = $cloudns->Helper->getPost('editRecordTtl');
			$status = 1;
			
			if ($recordType == 'SRV') {
				$settings['priority'] = $cloudns->Helper->getPost('editRecordPriority');
				$settings['weight'] = $cloudns->Helper->getPost('editRecordWeight');
				$settings['port'] = $cloudns->Helper->getPost('editRecordPort');
			} else if ($recordType == 'MX') {
				$settings['priority'] = $cloudns->Helper->getPost('editRecordPriority');
			} else if ($recordType == 'WR') {
				$settings['frame'] = $cloudns->Helper->getPost('editRecordWRFrame');
				$settings['frame-title'] = $cloudns->Helper->getPost('editRecordWRFrameTitle');
				$settings['frame-description'] = $cloudns->Helper->getPost('editRecordWRFrameDescription');
				$settings['frame-keywords'] = $cloudns->Helper->getPost('editRecordWRFrameKeywords');
				$settings['save-path'] = $cloudns->Helper->getPost('editRecordWRSavePath');
				$settings['redirect-type'] = $cloudns->Helper->getPost('wr_type');
			} else if ($recordType == 'RP') {
				$settings['mail'] = $cloudns->Helper->getPost('editRecordMail');
				$settings['txt'] = $cloudns->Helper->getPost('editRecordTxt');
			} else if ($recordType == 'SSHFP') {
				$settings['algorithm'] = $cloudns->Helper->getPost('algorithm');
				$settings['fptype'] = $cloudns->Helper->getPost('fp_type');
			} else if ($recordType == 'NAPTR') {
				$settings['order'] = $cloudns->Helper->getPost('editRecordOrder');
				$settings['pref'] = $cloudns->Helper->getPost('editRecordPref');
				$settings['flag'] = $cloudns->Helper->getPost('flag');
				$settings['params'] = $cloudns->Helper->getPost('editRecordParams');
				$settings['regexp'] = $cloudns->Helper->getPost('editRecordRegexp');
				$settings['replace'] = $cloudns->Helper->getPost('editRecordReplace');
			}
			
			$templateVariables = $cloudns->Actions->doEditRecord($zone, $record_id, $settings, $status, $recordType);
			$templateFile = 'templates/records.tpl';
				
		} elseif ($requestedAction == 'add-new-zone') {
			$version = explode('.', $params['whmcsVersion']);
			$templateVariables = array(
				'serversList' => $cloudns->Servers->getAvailableServers(),
				'cloudAction' => 'add-new-zone',
				'version'=>$version[0],
				'theme' => $params['clientareatemplate'],
			);
			
			$templateFile = 'templates/add-new-zone.tpl';
			
		} elseif ($requestedAction == 'soa-settings') {
			$templateVariables = $cloudns->Actions->getSOASettings($zone);
			$templateFile = 'templates/soa.tpl';
			
		} elseif ($requestedAction == 'edit-soa-settings') {
			$soa = array();
			$soa['primaryNS'] = $cloudns->Helper->getPost('primaryNS');
			$soa['adminMail'] = $cloudns->Helper->getPost('adminMail');
			$soa['refresh'] = $cloudns->Helper->getPost('refresh');
			$soa['retry'] = $cloudns->Helper->getPost('retry');
			$soa['expire'] = $cloudns->Helper->getPost('expire');
			$soa['defaultTTL'] = $cloudns->Helper->getPost('defaultTTL');
			$soaAction = 'do_save';
			if (isset($_POST['do_reset'])) {
				$soaAction = 'do_reset';
			}
			$templateVariables = $cloudns->Actions->editSOA($zone, $soa, $soaAction);
			$templateFile = 'templates/soa.tpl';
			
		} elseif ($requestedAction == 'update-status') {
			$templateVariables = $cloudns->Actions->getUpdateStatus($zone);
			$templateFile = 'templates/update-status.tpl';
			
		} elseif ($requestedAction == 'update') {
			$templateFile = 'templates/update-status.tpl';
			$templateVariables = $cloudns->Actions->updateZone($zone);
			
		} elseif ($requestedAction == 'import') {
			$templateFile = 'templates/import.tpl';
			$version = explode('.', $params['whmcsVersion']);
			$response = '';
			$templateVariables = array(
				'pagetitle' => 'Import records to '.$zone,
				'response' => $response,
				'cloudAction' => 'import',
				'version'=>$version[0],
				'theme' => $params['clientareatemplate'],
			);
			
		} elseif ($requestedAction == 'import-records') {
			$fileType = $cloudns->Helper->getPost('type');
			if (!in_array($fileType, array('bind', 'tinydns'))) {
				$templateVariables = array(
					'pagetitle' => 'Import records to '.$zone,
					'response' => 'Invalid DNS zone file type',
					'cloudAction' => 'import',
				);
			} else {
				$delete = $cloudns->Helper->getPost('delete');
				if (!in_array($delete, array(0, 1))) {
					$templateVariables = array(
						'pagetitle' => 'Import records to '.$zone,
						'response' => 'Invalid delete parameter',
						'cloudAction' => 'import',
					);
				}
				$format = $cloudns->Helper->getPost('format');
				$records = html_entity_decode($cloudns->Helper->getPost('recordsList'));
				
				$templateVariables = $cloudns->Actions->importRecords($zone, $records, $delete, $format);
			}
			$templateFile = 'templates/import.tpl';
			
		} elseif ($requestedAction == 'statistics') {
			$templateVariables = $cloudns->Actions->getStats($zone, $cloudns->Helper->getGet('date', ''));
			$templateFile = 'templates/statistics.tpl';
			
		} else {
			$templateFile = 'templates/zones.tpl';
			$templateVariables = $cloudns->Actions->getZones();
		}

		// adding the below keys, because they are used everywhere and there is no need to add them in each action
		
		$version = explode('.', $params['whmcsVersion']);
		$templateVariables['serviceid'] = $params['serviceid'];
		$templateVariables['zone'] = $zone;
		$templateVariables['version']=$version[0];
		$templateVariables['theme'] = $params['clientareatemplate'];
		
		// returning the template
		return array(
			'tabOverviewReplacementTemplate' => $templateFile,
			'vars' => $templateVariables,
			'cloudAction' => $requestedAction,
		);
		
	} catch (Exception $e) {
		// Record the error in WHMCS's module log.
		logModuleCall(
			'cloudns',
			__FUNCTION__,
			$params,
			$e->getMessage(),
			$e->getTraceAsString()
		);
		$version = explode('.', $params['whmcsVersion']);
		// In an error condition, display an error page.
		return array(
			'tabOverviewReplacementTemplate' => 'zone-error.tpl',
			'templateVariables' => array(
				'zone' => $zone,
				'response' => array('status'=>'error', 'description'=>$e->getMessage()),
				'usefulErrorHelper' => $e->getMessage(),
				'version'=>$version[0],
				'theme' => $params['clientareatemplate'],
				'cloudAction' => 'error',
			),
		);
	}
}

