<?php

class Cloudns_Core {
	/**
	 * Objects manager to use a single instance in all scripts
	 * 
	 * @Author	ClouDNS.net <support@cloudns.net>
	 * @Copyright	ClouDNS
	 * @Package	core
	 *
	 * @param Actions $Actions
	 * @param Api $Api
	 * @param Configuration Configuration
	 * @param Controller $Controller
	 * @param Database $Database 
	 * @param Helper $Helper
	 * @param Records $Records
	 * @param Servers $Servers
	 * @param Slave $Slave
	 * @param SOA $SOA
	 * @param Statistics $Statistics
	 * @param Zones $Zones 
	 * 
	 */
	
	
	/**
	 * @var Cloudns_Core
	 */
	private static $instance;
	
	/**
	 * Object intances
	 * @var array
	 */
	private $instances;
	
	protected $params;
	
	public function __construct($params) {
		$version = explode('.', $params['whmcsVersion']);
		$this->params = array(
			'version' => $params['whmcsVersion'],
			'serviceid'=>$params['serviceid'],
			'shortVersion'=>$version[0],
			'theme'=>$params['clientareatemplate'],
			'zonesLimit'=>$params['configoption1'],
			'availableServers'=>$params['configoption2'],
		);
	}
	
	/**
	 * @return Core 
	 */
	public static function inst ($params=array()) {
		if (is_null(self::$instance)) {
			self::$instance = new Cloudns_Core($params);
		}
		return self::$instance;
	}
	
	public function __get ($name) {
		$name = strtolower($name);
		if (!isset($this->instances[$name])) {
			if (!class_exists($name)) {
				$file = dirname(__FILE__) . "/" . $name.'.php';
				if (file_exists($file)) {
					include $file;
				} else {
					throw new Exception("The file ".$file." does not exists");
				}
			}
			$classname = "Cloudns_{$name}";
			$this->instances[$name] = new $classname($this->params);
		}
		
		return $this->instances[$name];
	}
}

